package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;
import com.qmth.distributed.print.business.bean.dto.professional.MatrixDto;
import com.qmth.distributed.print.business.bean.dto.professional.MatrixRequirementDto;
import com.qmth.distributed.print.business.bean.dto.professional.MatrixSubRequirementDto;
import com.qmth.distributed.print.business.entity.TPCourse;
import com.qmth.distributed.print.business.entity.TPMatrix;
import com.qmth.distributed.print.business.entity.TPProfessional;
import com.qmth.distributed.print.business.entity.TPRequirement;
import com.qmth.distributed.print.business.mapper.TPMatrixMapper;
import com.qmth.distributed.print.business.service.TPCourseService;
import com.qmth.distributed.print.business.service.TPMatrixService;
import com.qmth.distributed.print.business.service.TPProfessionalService;
import com.qmth.distributed.print.business.service.TPRequirementService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 专业认证毕业要求支撑矩阵 服务实现类
 * </p>
 *
 * @author xf
 * @since 2024-04-22
 */
@Service
public class TPMatrixServiceImpl extends ServiceImpl<TPMatrixMapper, TPMatrix> implements TPMatrixService {

    @Resource
    private TPProfessionalService tpProfessionalService;
    @Resource
    private TPCourseService tpCourseService;
    @Resource
    private TPRequirementService tpRequirementService;

    @Override
    public List<MatrixDto> listMatrix(Long professionalId) {
        List<TPMatrix> tpMatrixList = this.list(new QueryWrapper<TPMatrix>().lambda().eq(TPMatrix::getProfessionalId, professionalId));
        if (CollectionUtils.isNotEmpty(tpMatrixList)) {
            List<MatrixDto> matrixDtoList = new ArrayList<>();
            Map<Long, List<TPMatrix>> courseMap = tpMatrixList.stream().collect(Collectors.groupingBy(TPMatrix::getCourseId));
            for (Map.Entry<Long, List<TPMatrix>> entry : courseMap.entrySet()) {
                TPCourse tpCourse = tpCourseService.getById(entry.getKey());
                MatrixDto matrixDto = new MatrixDto();
                matrixDto.setCourseCode(tpCourse.getCourseCode());
                matrixDto.setCourseName(tpCourse.getCourseName());
                List<MatrixRequirementDto> matrixRequirementDtoList = new ArrayList<>();
                Map<Long, List<TPMatrix>> requirementMap = entry.getValue().stream().collect(Collectors.groupingBy(TPMatrix::getRequirementId));
                for (Map.Entry<Long, List<TPMatrix>> requirementEntry : requirementMap.entrySet()) {
                    TPRequirement tpRequirement = tpRequirementService.getById(requirementEntry.getKey());
                    MatrixRequirementDto matrixRequirementDto = new MatrixRequirementDto();
                    matrixRequirementDto.setName(tpRequirement.getName());
                    // 排序
                    List<MatrixSubRequirementDto> matrixSubRequirementDtoList = requirementEntry.getValue().stream().sorted(Comparator.comparing(TPMatrix::getSubName)).map(tpMatrix -> {
                        MatrixSubRequirementDto matrixSubRequirementDto = new MatrixSubRequirementDto();
                        matrixSubRequirementDto.setId(tpMatrix.getId());
                        matrixSubRequirementDto.setName(tpMatrix.getSubName());
                        matrixSubRequirementDto.setContent(tpMatrix.getContent());
                        return matrixSubRequirementDto;
                    }).collect(Collectors.toList());
                    matrixRequirementDto.setSubRequirements(matrixSubRequirementDtoList);
                    matrixRequirementDtoList.add(matrixRequirementDto);
                }
                matrixDto.setRequirements(matrixRequirementDtoList);
                matrixDtoList.add(matrixDto);
            }
            return matrixDtoList;
        }
        return new ArrayList<>();
    }

    @Override
    public boolean saveMatrix(TPMatrix tpMatrix) {
        UpdateWrapper<TPMatrix> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(TPMatrix::getContent, tpMatrix.getContent())
                .eq(TPMatrix::getId, tpMatrix.getId());
        return this.update(updateWrapper);
    }

    @Override
    public void downloadMatrix(Long professionalId, HttpServletResponse response) {
        TPProfessional tpProfessional = tpProfessionalService.getById(professionalId);
        List<TPMatrix> tpMatrixList = this.list(new QueryWrapper<TPMatrix>().lambda().eq(TPMatrix::getProfessionalId, professionalId));
        if (CollectionUtils.isNotEmpty(tpMatrixList)) {
            List<DataMap> matrixMapList = new ArrayList<>();
            Map<Long, List<TPMatrix>> courseMap = tpMatrixList.stream().collect(Collectors.groupingBy(TPMatrix::getCourseId));
            List<String> columnNameList = Arrays.asList("课程名称");
            // 是否添加表头列
            boolean addColumn = true;
            for (Map.Entry<Long, List<TPMatrix>> entry : courseMap.entrySet()) {
                TPCourse tpCourse = tpCourseService.getById(entry.getKey());
                DataMap dataMap = new DataMap();
                dataMap.put("courseName", tpCourse.getCourseName());
                Map<Long, List<TPMatrix>> requirementMap = entry.getValue().stream().collect(Collectors.groupingBy(TPMatrix::getRequirementId));
                for (Map.Entry<Long, List<TPMatrix>> requirementEntry : requirementMap.entrySet()) {
                    TPRequirement tpRequirement = tpRequirementService.getById(requirementEntry.getKey());
                    // 排序
                    boolean finalAddColumn = addColumn;
                    requirementEntry.getValue().stream().sorted(Comparator.comparing(TPMatrix::getSubName)).forEach(tpMatrix -> {
                        if (finalAddColumn) {
                            columnNameList.add(tpMatrix.getSubName());
                        }
                        dataMap.put(tpMatrix.getSubName(), tpMatrix.getContent() == null ? "" : tpMatrix.getContent().toString());
                    });
                }
                addColumn = false;
            }
            String[] columnNames = columnNameList.toArray(new String[0]);
            try {
                log.debug("导出Excel开始...");
                response.setHeader("Content-Disposition", "inline;filename=" + URLEncoder.encode(tpProfessional.getName() + "毕业要求支撑矩阵", SystemConstant.CHARSET_NAME) + ".xlsx");
                response.setContentType("application/vnd.ms-excel");
                ServletOutputStream outputStream = response.getOutputStream();
                ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
                writer.writeDataMaps("毕业要求支撑矩阵", null, columnNames, matrixMapList.listIterator());
                writer.output(outputStream);
                outputStream.flush();
                outputStream.close();
                log.debug("导出Excel结束");
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    @Override
    public List<TPMatrix> listMatrixByCondition(Long professionalId, Long courseId, Long requirementId) {
        QueryWrapper<TPMatrix> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<TPMatrix> lambda = queryWrapper.lambda();
        lambda.eq(TPMatrix::getProfessionalId, professionalId);
        if (courseId != null) {
            lambda.eq(TPMatrix::getCourseId, courseId);
        }
        if (requirementId != null) {
            lambda.eq(TPMatrix::getRequirementId, requirementId);
        }
        return this.list(queryWrapper);
    }

    @Override
    public void updateMatrixForNodeCount(TPCourse tpCourse, TPRequirement tpRequirement) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        List<TPMatrix> matrixList = this.listMatrixByCondition(tpRequirement.getProfessionalId(), tpCourse.getId(), tpRequirement.getId());
        List<TPMatrix> tpMatrixAddList = new ArrayList<>();
        List<Long> tpMatrixRemoveList = new ArrayList<>();
        // 新增
        if (CollectionUtils.isEmpty(matrixList)) {
            if (tpRequirement.getNodeCount() == null) {
                TPMatrix tpMatrix = new TPMatrix();
                tpMatrix.insertInfo(sysUser.getId());
                tpMatrix.setProfessionalId(tpRequirement.getProfessionalId());
                tpMatrix.setCourseId(tpCourse.getId());
                tpMatrix.setRequirementId(tpRequirement.getId());
                tpMatrixAddList.add(tpMatrix);
            }
        }
        // 修改
        else {
            if (tpRequirement.getNodeCount() == null) {
                for (int i = 0; i < matrixList.size(); i++) {
                    tpMatrixRemoveList.add(matrixList.get(i).getId());
                }
                TPMatrix tpMatrix = new TPMatrix();
                tpMatrix.insertInfo(sysUser.getId());
                tpMatrix.setProfessionalId(tpRequirement.getProfessionalId());
                tpMatrix.setCourseId(tpCourse.getId());
                tpMatrix.setRequirementId(tpRequirement.getId());
                tpMatrixAddList.add(tpMatrix);
            } else {
                if (matrixList.size() == 1) {
                    if(matrixList.get(0).getSubName() == null){
                        tpMatrixRemoveList.add(matrixList.get(0).getId());

                        for (int i = 1; i <= tpRequirement.getNodeCount(); i++) {
                            TPMatrix tpMatrix = new TPMatrix();
                            tpMatrix.insertInfo(sysUser.getId());
                            tpMatrix.setProfessionalId(tpRequirement.getProfessionalId());
                            tpMatrix.setCourseId(tpCourse.getId());
                            tpMatrix.setRequirementId(tpRequirement.getId());
                            tpMatrix.setSubName(tpRequirement.getSortNum() + "-" + i);
                            tpMatrixAddList.add(tpMatrix);
                        }
                    } else {
                        if (tpRequirement.getNodeCount() > matrixList.size()) {
                            for (int i = matrixList.size() + 1; i <= tpRequirement.getNodeCount(); i++) {
                                TPMatrix tpMatrix = new TPMatrix();
                                tpMatrix.insertInfo(sysUser.getId());
                                tpMatrix.setProfessionalId(tpRequirement.getProfessionalId());
                                tpMatrix.setCourseId(tpCourse.getId());
                                tpMatrix.setRequirementId(tpRequirement.getId());
                                tpMatrix.setSubName(tpRequirement.getSortNum() + "-" + i);
                                tpMatrixAddList.add(tpMatrix);
                            }
                        } else if (tpRequirement.getNodeCount() < matrixList.size()) {
                            for (int i = tpRequirement.getNodeCount(); i <= matrixList.size() - 1; i++) {
                                tpMatrixRemoveList.add(matrixList.get(i).getId());
                            }
                        }
                    }
                } else {
                    if (tpRequirement.getNodeCount() > matrixList.size()) {
                        for (int i = matrixList.size() + 1; i <= tpRequirement.getNodeCount(); i++) {
                            TPMatrix tpMatrix = new TPMatrix();
                            tpMatrix.insertInfo(sysUser.getId());
                            tpMatrix.setProfessionalId(tpRequirement.getProfessionalId());
                            tpMatrix.setCourseId(tpCourse.getId());
                            tpMatrix.setRequirementId(tpRequirement.getId());
                            tpMatrix.setSubName(tpRequirement.getSortNum() + "-" + i);
                            tpMatrixAddList.add(tpMatrix);
                        }
                    } else if (tpRequirement.getNodeCount() < matrixList.size()) {
                        for (int i = tpRequirement.getNodeCount(); i <= matrixList.size() - 1; i++) {
                            tpMatrixRemoveList.add(matrixList.get(i).getId());
                        }
                    }
                }
            }
        }
        if (CollectionUtils.isNotEmpty(tpMatrixAddList)) {
            this.saveOrUpdateBatch(tpMatrixAddList);
        }
        if (CollectionUtils.isNotEmpty(tpMatrixRemoveList)) {
            this.removeByIds(tpMatrixRemoveList);
        }
    }

    @Override
    public void removeMatrixByCondition(Long professionalId, Long courseId, Long requirementId) {
        UpdateWrapper<TPMatrix> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<TPMatrix> lambda = updateWrapper.lambda();
        if (professionalId != null) {
            lambda.eq(TPMatrix::getProfessionalId, professionalId);
        }
        if (courseId != null) {
            lambda.eq(TPMatrix::getCourseId, courseId);
        }
        if (requirementId != null) {
            lambda.eq(TPMatrix::getRequirementId, requirementId);
        }
        this.remove(updateWrapper);
    }
}
